/*-----------------------------------------------------------------------------
  Project:  QRE
  File:     qre.h

  Copyright (C)2010 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  $Revision: 1.10 $
 *----------------------------------------------------------------------------*/
#ifndef MW_QRE_QRE_H_
#define MW_QRE_QRE_H_

//==============================================================================

#include <nn.h>

#ifdef __cplusplus

namespace mw {
namespace qre {

/** 誤り訂正レベル */
typedef enum EccLevel
{
    ECC_LEVEL_L = 0,
    ECC_LEVEL_M,
    ECC_LEVEL_Q,
    ECC_LEVEL_H
} EccLevel;

/** QRコード情報 */
typedef struct EncodeData
{
    u32 size;           /**< バイナリQRデータのバイト数 */
    u8* data;           /**< バイナリQRデータへのポインタ */
    u32 version;        /**< フォーマットバージョン （0〜40, 0:自動） */
    u32 total;          /**< 分割数 （0〜16, 0:自動） */
    u32 cell_size;      /**< セルのサイズ */
    EccLevel ecc_level; /**< 誤り訂正レベル  */
    NN_PADDING3;
} EncodeData;

/** 位置情報 */
typedef struct Rect
{
    s32 top;            /**< 上限 */
    s32 left;           /**< 左 */
    s32 right;          /**< 右 */
    s32 bottom;         /**< 下限 */
} Rect;

/** RGBイメージ情報 */
typedef struct ImageInfo
{
    u16 width;          /**< 幅 */
    u16 height;         /**< 高さ */
    u8* rgb888Data;     /**< RGB画素データ */
    Rect imgPos;        /**< イメージ一座標 */
} ImageInfo;

//==============================================================================

/** @brief QRコード生成クラス */
class QREncoder
{
public:
    /** @brief コンストラクタ */
    QREncoder();
    /** @brief デストラクタ */
    virtual ~QREncoder();

    /**
     * @brief QRコード生成
     *
     * @param[in] qrInfo QRコード情報構造体
     * @retval    true   正常終了
     * @retval    false  処理失敗
     */
    bool Encode(EncodeData* qrInfo);

    /**
     * @brief デザインQRコード生成
     *
     * @param[in] qrInfo    QRコード情報構造体
     * @param[in] imageInfo QRコードに埋め込むイメージの情報
     * @retval    true      正常終了
     * @retval    false     処理失敗
     */
    bool EncodeWithImage(EncodeData* qrInfo, const ImageInfo* imageInfo);

    /**
     * @brief エラー情報取得
     *
     * @param[in] pMess エラー情報文字列格納用
     * @param[in] size  メッセージ
     * @retval    true  正常終了
     * @retval    false 処理失敗
     */
    bool GetErrorMessage(char* pMess, int size);

    /**
     * @brief 分割数取得
     *
     * @return 分割数
     */
    u32 GetDivCount();

    /**
     * @brief フォーマットバージョン取得
     *
     * @return フォーマットバージョン
     */
    u32 GetVersion();

    /**
     * @brief QRデータサイズ取得
     *
     * @param[in]  index 分割番号(0〜15)
     * @param[in]  isBmp 取得するデータの種類 true: BMPファイル false: イメージ情報
     * @return BMPファイルサイズ
     */
    u32 GetQRSize(u32 index = 0, bool isBmp = false);

    /**
     * @brief BMPデータ取得
     *
     * @param[out] buff  BMPデータバッファ
     * @param[in]  size  バッファサイズ
     * @param[in]  index 分割番号(0〜15)
     * @param[out] pos   イメージの位置
     * @retval     true  正常終了
     * @retval     false 処理失敗
     */
    bool GetQRBMPData(u8* buff, u32 size, u32 index = 0, mw::qre::Rect *pos = NULL);

    /**
     * @brief QRイメージ情報取得
     *
     * @param[out] info  QRイメージ情報
     * @param[in]  index 分割番号(0〜15)
     * @retval     true  正常終了
     * @retval     false 処理失敗
     */
    bool GetQRData(ImageInfo* info, u32 index = 0);

    /**
     * @brief ライブラリで使用するメモリ領域の登録
     *
     * @param[in] ptr   メモリ領域
     * @param[in] size  メモリ領域のサイズ
     * @retval    true  正常終了
     * @retval    false 処理失敗
     */
    bool InitMemory(void* ptr, u32 size);

    /**
     * @brief ライブラリで使用するメモリサイズの取得
     *
     * @param[in] data_size エンコードするデータのサイズ
     * @param[in] cell_size QRコードイメージの1フォーマットの幅 (or 高さ dot)
     * @param[in] count     QRコード分割数 (0 … データサイズに依存する)
     * @return              必要とするメモリサイズ
     */
    static u32 GetEncodeBufferSize(u32 data_size, u32 cell_size, u32 count = 0);

    /**
     * @brief デザインQR埋め込み可能最大イメージサイズ取得
     *
     * @param[in] version  QRバージョン
     * @param[in] cellSize 1セルのドット数
     * @return             埋め込み可能な最大イメージの一辺のサイズ
     */
    static u32 GetDesignQRImageSize(u32 version, u32 cellSize);
};

} // namespace qre
} // namespcae mw

#endif	// __cplusplus

//==============================================================================

#ifdef __cplusplus
# define MW_QRE_EXTERN_C extern "C"
#else
# define MW_QRE_EXTERN_C extern
#endif

/*!
 * @union mwqreQREncoder
 * @brief  QRエンコーダをあらわす C の共用体です。
 * @brief  対応するmw::qre::QREncoder を参照してください。
 */
typedef union mwqreQREncoder
{
	char buf[4];
	u32 alignment_holder;
} mwqreQREncoder;

/** 誤り訂正レベル */
typedef enum mwqreEccLevel
{
    MW_QRE_ECC_LEVEL_L = 0,
    MW_QRE_ECC_LEVEL_M,
    MW_QRE_ECC_LEVEL_Q,
    MW_QRE_ECC_LEVEL_H
} mwqreEccLevel;

/** QRコード情報 */
typedef struct mwqreEncodeData
{
    u32 size;                /**< バイナリQRデータのバイト数 */
    u8* data;                /**< バイナリQRデータへのポインタ */
    u32 version;             /**< フォーマットバージョン （0〜40, 0:自動） */
    u32 total;               /**< 分割数 （0〜16, 0:自動） */
    u32 cell_size;           /**< セルのサイズ */
    mwqreEccLevel ecc_level; /**< 誤り訂正レベル  */
    NN_PADDING3;
} mwqreEncodeData;

/** 位置情報 */
typedef struct mwqreRect
{
    s32 top;                 /**< 上限 */
    s32 left;                /**< 左 */
    s32 right;               /**< 右 */
    s32 bottom;              /**< 下限 */
} mwqreRect;

/** RGBイメージ情報 */
typedef struct mwqreImageInfo
{
    u16 width;               /**< 幅 */
    u16 height;              /**< 高さ */
    u8* rgb888Data;          /**< RGB画素データ */
    mwqreRect imgPos;        /**< イメージ一座標 */
} mwqreImageInfo;

/*!
 *  @brief 対応する C++ 関数 @ref mw::qre::QREncoder::Encode を参照してください。
 */
MW_QRE_EXTERN_C bool mwqreQREncoderEncode(mwqreQREncoder* this_, mwqreEncodeData* qrInfo);

/*!
 *  @brief 対応する C++ 関数 @ref mw::qre::QREncoder::EncodeWithImage を参照してください。
 */
MW_QRE_EXTERN_C bool mwqreQREncoderEncodeWithImage(mwqreQREncoder* this_, mwqreEncodeData* qrInfo, const mwqreImageInfo* imageInfo);

/*!
 *  @brief 対応する C++ 関数 @ref mw::qre::QREncoder::GetErrorMessage を参照してください。
 */
MW_QRE_EXTERN_C bool mwqreQREncoderGetErrorMessage(mwqreQREncoder* this_, char* pMess, int size);

/*!
 *  @brief 対応する C++ 関数 @ref mw::qre::QREncoder::GetDivCount を参照してください。
 */
MW_QRE_EXTERN_C u32 mwqreQREncoderGetDivCount(mwqreQREncoder* this_);

/*!
 *  @brief 対応する C++ 関数 @ref mw::qre::QREncoder::GetVersion を参照してください。
 */
MW_QRE_EXTERN_C u32 mwqreQREncoderGetVersion(mwqreQREncoder* this_);

/*!
 *  @brief 対応する C++ 関数 @ref mw::qre::QREncoder::GetQRSize を参照してください。
 */
MW_QRE_EXTERN_C u32 mwqreQREncoderGetQRSize(mwqreQREncoder* this_, u32 index, bool isBmp);

/*!
 *  @brief 対応する C++ 関数 @ref mw::qre::QREncoder::GetQRBMPData を参照してください。
 */
MW_QRE_EXTERN_C bool mwqreQREncoderGetQRBMPData(mwqreQREncoder* this_, u8* buff, u32 size, u32 index, mwqreRect* pos);

/*!
 *  @brief 対応する C++ 関数 @ref mw::qre::QREncoder::GetQRData を参照してください。
 */
MW_QRE_EXTERN_C bool mwqreQREncoderGetQRData(mwqreQREncoder* this_, mwqreImageInfo* info, u32 index);

/*!
 *  @brief 対応する C++ 関数 @ref mw::qre::QREncoder::InitMemory を参照してください。
 */
MW_QRE_EXTERN_C bool mwqreQREncoderInitMemory(mwqreQREncoder* this_, void* ptr, u32 size);

/*!
 *  @brief 対応する C++ 関数 @ref mw::qre::QREncoder::GetEncodeBufferSize を参照してください。
 */
MW_QRE_EXTERN_C u32 mwqreQREncoderGetEncodeBufferSize(u32 data_size, u32 cell_size, u32 count);

/*!
 *  @brief 対応する C++ 関数 @ref mw::qre::QREncoder::GetDesignQRImageSize を参照してください。
 */
MW_QRE_EXTERN_C u32 mwqreQREncoderGetDesignQRImageSize(u32 version, u32 cellSize);

#endif // MW_QRE_QRE_H_
// EOF
