/*-----------------------------------------------------------------------------
  Project:  QRE
  File:     WBMHandler.cpp

  Copyright (C)2010 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  $Revision: 1.6 $
 *----------------------------------------------------------------------------*/

/// @file
/// @brief ウィンドウズビットマップファイルヘッダ構造体操作クラス定義ファイル
/// @author Mario Ikeda, ISP.,Co.LTD.
/// @version 1.0

#include "WBMHandler.h"

/// @brief ビットマップヘッダ情報読み込みメソッド
///
/// 指定されたストリームよりビットマップのヘッダ情報を読み込む。
/// @param[in] is 情報を読み込む入力ストリーム
/// @retval 0 正常終了
/// @retval それ以外 処理失敗
int BitMapCoreHeader::read_header(ISTREAM &is)
{
    // サイズ情報のバッファサイズはヘッダの種類によって異なるので、
    // クラスごとのサイズ情報読み込みメソッドを呼び出す。
    int ret = read_sizeinfo(is);

    if (!ret) {
        // サイズ情報の読み込みに成功したら残りの情報も読み出す。
        ReadFile(is, &plane);
        ReadFile(is, &color_bits);
    }
    return ret;
}

/// @brief ビットマップヘッダ情報保存メソッド
///
/// 指定されたストリームにビットマップのヘッダ情報を保存する。
/// @param[out] os 情報を保存する出力ストリーム
/// @retval 0 正常終了
/// @retval それ以外 処理失敗
int BitMapCoreHeader::write_header(OSTREAM &os)
{
    WriteFile(os, &header_size);

    // サイズ情報のバッファサイズはヘッダの種類によって異なるので、
    // クラスごとのサイズ情報保存メソッドを呼び出す。
    int ret = write_sizeinfo(os);

    if (!ret) {
        // サイズ情報保存に成功したら残りの情報も保存する。
        WriteFile(os, &plane);
        WriteFile(os, &color_bits);
    }
    return ret;
}

/// @brief サイズ情報読み込みメソッド
///
/// サイズ情報のバッファサイズは、OS2 Version 1は2byte、それ以外は4byteなので、
/// ヘッダの種類によって読み込みメソッドを変える。
/// @param[in] is 情報を読み込む入力ストリーム
/// @retval 0 正常終了
/// @retval それ以外 処理失敗
int BitMapCoreHeader::read_sizeinfo(ISTREAM &is)
{
    uint16_t uwork;
    ReadFile(is, &uwork);
    width = static_cast<uint32_t>(uwork);
    int16_t swork;
    ReadFile(is, &swork);
    height = static_cast<int32_t>(swork);

    return 0;
}

/// @brief サイズ情報保存メソッド
///
/// 読み込みと同様の理由で、ヘッダの種類により情報保存関数をかえる。
/// @param[out] os 情報を保存する出力ストリーム
/// @retval 0 正常終了
/// @retval それ以外 処理失敗
int BitMapCoreHeader::write_sizeinfo(OSTREAM &os)
{
    uint16_t uwork = static_cast<uint16_t>(width);
    WriteFile(os, &uwork);
    int16_t swork = static_cast<int16_t>(height);
    WriteFile(os, &swork);

    return 0;
}

/// @brief ビットマップヘッダ情報読み込みメソッド
///
/// 指定されたストリームよりビットマップのヘッダ情報を読み込む。
/// @param[in] is 情報を読み込む入力ストリーム
/// @retval 0 正常終了
/// @retval それ以外 処理失敗
int BitMapInfoHeader::read_header(ISTREAM &is)
{
    int ret = BitMapCoreHeader::read_header(is);
    if (ret)
        return ret;

    // クラスメンバを保存する。
    ReadFile(is, &compress);
    ReadFile(is, &image_size);
    ReadFile(is, &horizontal_PPM);
    ReadFile(is, &vertical_PPM);
    ReadFile(is, &num_of_color);
    ReadFile(is, &important_color);

    return 0;
}

/// @brief ビットマップヘッダ情報保存メソッド
///
/// 指定されたストリームにビットマップのヘッダ情報を保存する。
/// @param[out] os 情報を保存する出力ストリーム
/// @retval 0 正常終了
/// @retval それ以外 処理失敗
int BitMapInfoHeader::write_header(OSTREAM &os)
{
    int ret = BitMapCoreHeader::write_header(os);
    if (ret)
        return ret;

    // クラスメンバを保存する。
    WriteFile(os, &compress);
    WriteFile(os, &image_size);
    WriteFile(os, &horizontal_PPM);
    WriteFile(os, &vertical_PPM);
    WriteFile(os, &num_of_color);
    WriteFile(os, &important_color);

    return 0;
}

/// @brief サイズ情報読み込みメソッド
///
/// BITMAPCOREHEADERクラスのメソッドと同様の理由で、
/// ヘッダの種類により情報読み込みメソッドを変える。
/// @param[in] is 情報を読み込む入力ストリーム
/// @retval 0 正常終了
/// @retval それ以外 処理失敗
int BitMapInfoHeader::read_sizeinfo(ISTREAM &is)
{
    ReadFile(is, &width);
    ReadFile(is, &height);

    return 0;
}

/// @brief サイズ情報保存メソッド
///
/// 読み込みと同様の理由で、ヘッダの種類により情報保存関数をかえる。
/// @param[out] os 情報を保存する出力ストリーム
/// @retval 0 正常終了
/// @retval それ以外 処理失敗
int BitMapInfoHeader::write_sizeinfo(OSTREAM &os)
{
    WriteFile(os, &width);
    WriteFile(os, &height);

    return 0;
}

/// @brief ビットマップ情報読み込みメソッド
///
/// 指定されたストリームよりビットマップの基本的な情報を読み込む。
/// @param[in] is 情報を読み込む入力ストリーム
/// @retval 0 正常終了
/// @retval それ以外 処理失敗
int BitMapFileHeader::read_header(ISTREAM &is)
{
    // クラスメンバの読み込み
    size_t read_size = sizeof(char) * sizeof(magic);
    ReadFile(is, &magic, read_size);
    if (magic[0] != 'B' || magic[1] != 'M')
        return -1;

    ReadFile(is, &size);
    ReadFile(is, &reserve_1);
    ReadFile(is, &reserve_2);
    ReadFile(is, &offset);

    // ビットマップヘッダサイズを取得
    uint32_t header_size;
    ReadFile(is, &header_size);

    if (core_header) delete core_header;
    // サイズに生成するクラスを合わせる
    switch (header_size) {
    case BHS_OS2_V1_SIZE:
        core_header = new BitMapCoreHeader();
        break;
    case BHS_WIN_V3_SIZE:
        core_header
            = reinterpret_cast<BitMapCoreHeader *>(new BitMapInfoHeader());
        break;
    default:
        core_header = NULL;
        break;
    }
    if (!core_header) {
        return -1;
    }
    int ret = core_header->read_header(is);

    // 正常終了しており、
    if (ret == 0) {
        color_map_size = offset - BITMAPFILEHEADER_SIZE - header_size;
        // カラーマップのサイズがあったら…
        if (color_map_size > 0) {
            if (color_map)
                delete [] color_map;
            color_map = new uint8_t[color_map_size];
            if (!color_map)
                return -1;

            ReadFile(is, color_map, color_map_size);
        }
    }
    return ret;
}

/// @brief ビットマップ情報保存メソッド
///
/// 指定されたストリームにビットマップの基本的な情報を保存する。
/// @param[out] os 情報を保存する出力ストリーム
/// @retval 0 正常終了
/// @retval それ以外 処理失敗
int BitMapFileHeader::write_header(OSTREAM &os)
{
    if (!core_header || (color_map_size && !color_map))
        return -1;

    // クラスメンバを保存する。
    WriteFile(os, magic, sizeof(char) * sizeof(magic));
    WriteFile(os, &size);
    WriteFile(os, &reserve_1);
    WriteFile(os, &reserve_2);
    WriteFile(os, &offset);

    // ビットマップヘッダを保存する。
    int ret = core_header->write_header(os);
    // 保存に成功して、カラーマップを保存する必要がある場合…
    if (ret == 0 && color_map_size)
        WriteFile(os, color_map, color_map_size);

    return ret;
}

/* EOF */
