/*-----------------------------------------------------------------------------
  Project:  QRE
  File:     WBMHandler.h

  Copyright (C)2010 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  $Revision: 1.6 $
 *----------------------------------------------------------------------------*/
#ifndef WBMHANDLER_H_
# define WBMHANDLER_H_

/// @file
/// @brief ウィンドウズビットマップファイルヘッダ構造体操作クラス宣言ファイル
///
/// See <a href="http://ja.wikipedia.org/wiki/Windows_bitmap">
/// http://ja.wikipedia.org/wiki/Windows_bitmap</a>
/// @author Mario Ikeda, ISP.,Co.LTD.
/// @version 1.0

# include <nn/fs.h>

typedef nn::fs::FileInputStream ISTREAM;
typedef nn::fs::FileOutputStream OSTREAM;

// 各種ビットマップヘッダサイズ
const uint32_t BITMAPFILEHEADER_SIZE = 14;    ///< bitmap file header
const uint32_t BHS_OS2_V1_SIZE = 12;        ///< OS2 Version 1
const uint32_t BHS_OS2_V2_SIZE = 64;        ///< OS2 Version 2
const uint32_t BHS_WIN_V3_SIZE = 40;        ///< Windows Version 3
const uint32_t BHS_WIN_V4_SIZE = 108;        ///< Windows Version 4
const uint32_t BHS_WIN_V5_SIZE = 124;        ///< Windows Version 5

/**
 * @brief ファイル読み込み関数
 *
 * プラットフォームに依存する（I/F、byte order等）を一カ所で吸収するため、
 * ラッパ関数を使用する。
 * @param[in] is        入力ファイル識別子
 * @param[in] ptr       入力するデータへのポインタ
 * @param[in] read_size 入力するバイト数（0の場合は、データの型に依存する）
 * @return              入力ファイル識別子
 */
template<typename T>
inline ISTREAM &ReadFile(ISTREAM &is, T *ptr, size_t read_size = 0)
{
    if (!read_size)
        read_size = sizeof(T);
    is.Read(reinterpret_cast<char *>(ptr), read_size);
    return is;
}

/**
 * @brief ファイル書き込み関数
 *
 * プラットフォームに依存する（I/F、byte order等）を一カ所で吸収するため、
 * ラッパ関数を使用する。
 * @param[in] os         出力ファイル識別子
 * @param[in] ptr        出力するデータへのポインタ
 * @param[in] write_size 出力するバイト数（0の場合は、データの型に依存する）
 * @return               出力ファイル識別子
 */
template<typename T>
inline OSTREAM &WriteFile(OSTREAM &os, T *ptr, size_t write_size = 0)
{
    if (!write_size)
        write_size = sizeof(T);
    os.Write(reinterpret_cast<char *>(ptr), write_size, false);
    return os;
}

/**
 * @brief 4 rounding
 * @param[in] i16 Number to round
 * @return        Rounded number
 */
template<typename T>
inline T round4(T n)
{
    return (n + (static_cast<T>(3))) & ~(static_cast<T>(3));
}

/// @brief ビットマップコアヘッダ構造体 (OS2 V1)
///
/// サイズ 12byte
class BitMapCoreHeader {
protected:
    uint32_t header_size;    ///< ヘッダサイズ
    uint32_t width;            ///< 画像の幅
    int32_t height;            ///< 画像の高さ
    uint16_t plane;            ///< プレーン
    uint16_t color_bits;    ///< 色ビット数

public:
    /// @brief コンストラクタ
    /// @param[in] l ヘッダサイズ
    ///          (派生クラスへのI/F引数、通常は必ずデフォルト引数を使う事)
    BitMapCoreHeader(const uint32_t l = BHS_OS2_V1_SIZE):
        header_size(l), width(0), height(0), plane(1), color_bits(24) {}
    /// @brief コンストラクタ
    ///
    /// 指定された入力ストリームよりヘッダ情報を読み込む。
    /// @param[in] is ヘッダ情報を参照する入力ストリーム
    /// @param[in] l ヘッダサイズ
    ///          (派生クラスへのI/F引数、通常は必ずデフォルト引数を使う事)
    BitMapCoreHeader(ISTREAM &is,
                     const uint32_t l = BHS_OS2_V1_SIZE): header_size(l) {
        read_header(is);
    }
    /// デストラクタ
    virtual ~BitMapCoreHeader() {}

    virtual int read_header(ISTREAM &);
    virtual int write_header(OSTREAM &);

    /// @brief 画像幅参照
    /// @return 画像幅
    uint32_t get_width() { return width; }
    /// @brief 画像の高さ参照
    /// @return 画像の高さ
    int32_t get_height() { return height; }
    /// @brief 画素一ピクセルのビット数参照
    /// @return 画素一ピクセルのビット数
    uint32_t get_bits() { return color_bits; }
    /// @brief 水平方向解像度参照
    /// @return 情報が無いので0を返す
    virtual uint32_t get_horizontal_PPM() { return 0; }
    /// @brief 鉛直方向解像度参照
    /// @return 情報が無いので0を返す
    virtual uint32_t get_vertical_PPM() { return 0; }
    /// @brief イメージデータサイズ参照
    /// @return イメージデータサイズ
    virtual uint32_t get_image_size() {
        return height * round4(width * color_bits / 8) * (height > 0 ? 1 : -1);
    }
    /// @brief 画像幅設定
    /// @param[in] w 画像幅
    /// @return 画像幅
    virtual uint32_t set_width(const uint32_t w) {
        width = w; return width;
    }
    /// @brief 画像の高さ設定
    /// @param[in] h 画像の高さ
    /// @return 画像の高さ
    virtual int32_t set_height(const int32_t h) {
        height = h; return height;
    }
    /// @brief 水平方向解像度設定
    /// @return 情報が無いので0を返す
    virtual uint32_t set_horizontal_PPM(const uint32_t) { return 0; }
    /// @brief 鉛直方向解像度設定
    /// @return 情報が無いので0を返す
    virtual uint32_t set_vertical_PPM(const uint32_t) { return 0; }

protected:
    virtual int read_sizeinfo(ISTREAM &);
    virtual int write_sizeinfo(OSTREAM &);
};

/// @brief ビットマップインフォヘッダ構造体 (Windows v3)
///
/// サイズ 40byte
class BitMapInfoHeader: public BitMapCoreHeader {
protected:
    uint32_t compress;            ///< 圧縮 (未サポート)
    uint32_t image_size;        ///< 画像データサイズ
    uint32_t horizontal_PPM;    ///< 水平方向解像度 (pixel per meter)
    uint32_t vertical_PPM;        ///< 鉛直方向解像度 (pixel per meter)
    uint32_t num_of_color;        ///< 使用する色数
    uint32_t important_color;    ///< 重要な色数

public:
    /// @brief コンストラクタ
    /// @param[in] l ヘッダサイズ
    ///          (派生クラスへのI/F引数、通常は必ずデフォルト引数を使う事)
    BitMapInfoHeader(const uint32_t l = BHS_WIN_V3_SIZE):
        BitMapCoreHeader(l), compress(0), image_size(0),
        horizontal_PPM(2834), vertical_PPM(2834),
        num_of_color(0), important_color(0) {}
    /// @brief コンストラクタ
    ///
    /// 指定された入力ストリームよりヘッダ情報を読み込む。
    /// @param[in] is ヘッダ情報を参照する入力ストリーム
    /// @param[in] l ヘッダサイズ
    ///          (派生クラスへのI/F引数、通常は必ずデフォルト引数を使う事)
    BitMapInfoHeader(ISTREAM &is, const uint32_t l = BHS_WIN_V3_SIZE):
        BitMapCoreHeader(is, l) {

        read_header(is);
    }
    /// デストラクタ
    virtual ~BitMapInfoHeader() {}

    virtual int read_header(ISTREAM &);
    virtual int write_header(OSTREAM &);

    /// @brief 水平方向解像度参照
    /// @return 解像度 (Pixel per meter)
    virtual uint32_t get_horizontal_PPM() { return horizontal_PPM; }
    /// @brief 鉛直方向解像度参照
    /// @return 解像度 (Pixel per meter)
    virtual uint32_t get_vertical_PPM() { return vertical_PPM; }
    /// @brief イメージデータサイズ参照
    /// @return イメージデータサイズ
    virtual uint32_t get_image_size() { return image_size; }
    /// @brief 水平方向解像度設定
    /// @param[in] p 設定解像度
    /// @return 解像度 (Pixel per meter)
    virtual uint32_t set_horizontal_PPM(const uint32_t p) {
        horizontal_PPM = p;
        return horizontal_PPM;
    }
    /// @brief 鉛直方向解像度設定
    /// @param[in] p 設定解像度
    /// @return 解像度 (Pixel per meter)
    virtual uint32_t set_vertical_PPM(const uint32_t p) {
        vertical_PPM = p;
        return vertical_PPM;
    }

    /// @brief 画像幅設定
    /// @param[in] w 画像幅
    /// @return 画像幅
    virtual uint32_t set_width(const uint32_t w) {
        BitMapCoreHeader::set_width(w);
        recalc_size();
        return width;
    }
    /// @brief 画像の高さ設定
    /// @param[in] h 画像の高さ
    /// @return 画像の高さ
    virtual int32_t set_height(const int32_t h) {
        BitMapCoreHeader::set_height(h);
        recalc_size();
        return height;
    }

protected:
    virtual int read_sizeinfo(ISTREAM &);
    virtual int write_sizeinfo(OSTREAM &);

private:
    /// @brief ファイルサイズの再計算
    ///
    /// 保持しているサイズ情報を元に、ファイルサイズを再計算する。
    /// @retval 0 設定成功
    /// @retval それ以外 処理失敗
    int recalc_size()
    {
        image_size = BitMapCoreHeader::get_image_size();
        return 0;
    }
};

/// @brief ビットマップファイルヘッダ構造体
///
/// +----------------+--------------------------+----------------+--<br>
/// |2byte 識別子"BM"|4byte ファイルサイズ(byte)|2byte 予約領域１|…<br>
/// +----------------+--------------------------+----------------+--<br>
/// <br>
/// --+----------------+----------------------------------------+<br>
/// …|2byte 予約領域２|4byte ビットマップデータの開始位置(byte)|<br>
/// --+----------------+----------------------------------------+<br>
class BitMapFileHeader {
    NN_PADDING2;
    char magic[2];                    ///< 識別子 "BM"
    uint32_t size;                    ///< ファイルサイズ
    uint16_t reserve_1;                ///< 予約領域１
    uint16_t reserve_2;                ///< 予約領域２
    uint32_t offset;                ///< ビットマップデータの開始位置
    BitMapCoreHeader *core_header;    ///< ビットマップコアヘッダ
    uint8_t *color_map;                ///< indexカラー用カラーマップ
    uint32_t color_map_size;        ///< カラーマップサイズ
public:
    /// @brief コンストラクタ
    ///
    /// ポインタ領域の初期化処理を行う。
    /// とりあえず一般的なヘッダで初期化。
    BitMapFileHeader():
        size(BITMAPFILEHEADER_SIZE + BHS_WIN_V3_SIZE),
        reserve_1(0), reserve_2(0),
        offset(BITMAPFILEHEADER_SIZE + BHS_WIN_V3_SIZE),
        core_header(NULL), color_map(NULL), color_map_size(0) {
        magic[0] = 'B';
        magic[1] = 'M';
        core_header 
            = reinterpret_cast<BitMapCoreHeader *>(new BitMapInfoHeader());
    }
    /// @brief コンストラクタ
    ///
    /// 指定された入力ストリームよりヘッダ情報を読み込む。
    /// @param[in] is ヘッダ情報を参照する入力ストリーム
    BitMapFileHeader(ISTREAM &is):
        core_header(NULL), color_map(NULL) {

        read_header(is);
    }
    /// @brief デストラクタ
    ///
    /// 割り当てられたメモリ領域を解放する。
    virtual ~BitMapFileHeader() {
        if (core_header) delete core_header;
        if (color_map) delete [] color_map;
    }

    int read_header(ISTREAM &);
    int write_header(OSTREAM &);

    /// @brief 画像幅参照
    /// @return 画像幅
    uint32_t get_width() {
        return (core_header) ? core_header->get_width(): 0;
    }
    /// @brief 画像の高さ参照
    /// @return 画像の高さ
    int32_t get_height() {
        return (core_header) ? core_header->get_height(): 0;
    }
    /// @brief 画素一ピクセルのビット数参照
    /// @return 画素一ピクセルのビット数
    uint32_t get_bits() {
        return (core_header) ? core_header->get_bits(): 0;
    }
    /// @brief 水平方向解像度参照
    /// @return 解像度 (Pixel per meter)
    uint32_t get_horizontal_PPM() {
        return (core_header) ? core_header->get_horizontal_PPM(): 0;
    }
    /// @brief 鉛直方向解像度参照
    /// @return 解像度 (Pixel per meter)
    uint32_t get_vertical_PPM() {
        return (core_header) ? core_header->get_vertical_PPM(): 0;
    }

    /// @brief 画像幅設定
    /// @param[in] w 画像幅
    /// @retval 0 設定成功
    /// @retval それ以外 処理失敗
    int set_width(const uint32_t w) {
        if (!core_header)
            return -1;
        core_header->set_width(w);
        return recalc_size();
    }
    /// @brief 画像の高さ設定
    /// @param[in] h 画像の高さ
    /// @retval 0 設定成功
    /// @retval それ以外 処理失敗
    int set_height(const int32_t h) {
        if (!core_header)
            return -1;
        core_header->set_height(h);
        return recalc_size();
    }

    /// @brief 水平方向解像度設定
    /// @param[in] p 設定解像度
    /// @retval 0 処理失敗
    /// @retval それ以外 設定した解像度
    uint32_t set_horizontal_PPM(const uint32_t p) {
        if (!core_header)
            return 0;
        return core_header->set_horizontal_PPM(p);
    }
    /// @brief 鉛直方向解像度設定
    /// @param[in] p 設定解像度
    /// @retval 0 処理失敗
    /// @retval それ以外 設定した解像度
    uint32_t set_vertical_PPM(const uint32_t p) {
        if (!core_header)
            return 0;
        return core_header->set_vertical_PPM(p);
    }

    uint32_t get_header_size() {
        return offset;
    }

    uint32_t get_image_size() {
        return size - offset;
    }

private:
    /// @brief ファイルサイズの再計算
    ///
    /// 保持しているサイズ情報を元に、ファイルサイズを再計算する。
    /// @retval 0 設定成功
    /// @retval それ以外 処理失敗
    int recalc_size() {
        // ファイルサイズはビットマップデータの開始位置に
        // イメージサイズを足したもの
        size = static_cast<uint32_t>(offset + core_header->get_image_size());
        return 0;
    }
};

#endif // WBMHANDLER_H_
/* EOF */
